#include "global.h"


void InitReportsSort(void) {
    char sortset[URLSIZE];
    const char *sortorder;


    /* Fill reports sorting description, so we will not do this many times for every UFR, UR, UDR and LFR */
    if (islower((int)cfg.index_sort)) sortorder = loc->Asc;
    else sortorder = loc->Desc;
    switch (cfg.index_sort) {
	case 'd':
	case 'D':
		 SAsnprintf(sortset, sizeof(sortset), "%s (%s)", loc->GenerationD, sortorder);
		 break;
	case 'n':
	case 'N':
		 SAsnprintf(sortset, sizeof(sortset), "%s, %s (%s, %s)", loc->Configuration, loc->GenerationD, sortorder, loc->Desc);
		 break;
	case 'b':
	case 'B':
		 SAsnprintf(sortset, sizeof(sortset), "%s, %s (%s, %s)", loc->Bytes, loc->GenerationD, sortorder, loc->Desc);
		 break;
	case 'u':
	case 'U':
		 SAsnprintf(sortset, sizeof(sortset), "%s, %s (%s, %s)", loc->Users, loc->GenerationD, sortorder, loc->Desc);
		 break;
    }
    rcfg.IRsort = SAstrdup(sortset);

    SAsnprintf(sortset, sizeof(sortset), "%s, %s (%s, %s)", loc->URL, loc->DnT, loc->Asc, loc->Asc);
    rcfg.UFRsort = SAstrdup(sortset);
    SAsnprintf(sortset, sizeof(sortset), "%s (%s)", loc->Bytes, loc->Desc);
    rcfg.URsort = SAstrdup(sortset);
    SAsnprintf(sortset, sizeof(sortset), "%s (%s)", loc->DsT, loc->Asc);
    rcfg.UDRsort = SAstrdup(sortset);
    SAsnprintf(sortset, sizeof(sortset), "%s, %s (-, %s)", loc->User, loc->DnT, loc->Asc);
    rcfg.LFRsort = SAstrdup(sortset);
}


void InitReports(void) {
    short int i;
    char tfname[FILENAMESIZE];


    /* Prepare cache directory */
    SAsnprintf(tfname, sizeof(tfname), "%s/r%ld", cfg.cache_directory, (long int)getpid());
    cfg.cache_directory = SAstrdup(tfname);
    SArm(cfg.cache_directory);
    if (mkdir(cfg.cache_directory, 0700)) SAexit("%s. %s", cfg.cache_directory, strerror(errno));

    /* LF_UNS: open */
    for (i = 0; i < icfg.LFcount; i++) {
	FormFnames(&LF[i].fnames, "%s/lf_%hd", cfg.cache_directory, i);
	LF[i].file = SAfopenI("w", LF[i].fnames.uns);
	LF[i].all.count = 0;
    }

    /* Replace localisation strings */
    if (LogFormats[cfg.log_format].Class == MAIL) {
	loc->User = loc->Sender;
	loc->Connections = loc->Messages;
	loc->URL = loc->Recipient;
	loc->Users = loc->Senders;
	loc->TopUsers = loc->Senders;
	loc->TopSites = loc->Recipients;
    }
    InitReportsSort();
    if (cfg.top_sites_report) FormFnames(&rcfg.TSRfnames, "%s/%s", cfg.cache_directory, icfg.TSRname);

    /* Clear efficiency data */
    if (cfg.server_efficiency_report) memset(ec, 0, sizeof(ec));
}


void GenerateReports(void) {
    short int i, j;
    char tfname[FILENAMESIZE];
    FILE *DF;


    /* Create reports directory */
    j = SAsnprintf(tfname, sizeof(tfname), "%s/%s-", cfg.target_directory, icfg.Period);
    FixFileName(&tfname[j - strlen(icfg.Period)]);

    for (i = 1; ; i++) {
	SAsnprintf(&tfname[j], sizeof(tfname) - j, "%hd", i);
	if (mkdir(tfname, 0755)) {
	    if (errno != EEXIST) SAexit("%s. %s", tfname, strerror(errno));
	} else break;
    }

    /* Save reports directory */
    icfg.ReportDir = SAstrdup(tfname);

    if (cfg.reports_privacy_mode) InitPrivacy();
    ReportLocalFilter();
    if (ccCount) {
	ReportTopUsers();
	if (cfg.users_report || cfg.top_sites_report) ReportUsers();
	if (cfg.email_address != NULL) ReportEmail();
	if (cfg.top_sites_report) ReportTopSites();
    }
    if (cfg.server_efficiency_report) ReportProxyEfficiency();
    ReportList();

    /* DF: all */
    DF = SAfopen("w", "%s/%s", icfg.ReportDir, icfg.DFname);
    SAfprintf(DF, "%ld %ld\n%s\n%.0f\n%d\n", icfg.SDate, icfg.EDate, cfg.configuration_name, all[0].bytes, ccueCount);
    SAfclose(DF);

    ReportIndex();

    SArm(cfg.cache_directory);
}
