typedef struct Config {
    const char	*cache_directory;
    const char	*target_directory;
    const char	*reports_site_name;
    const char	*log_file;
    const char	*configuration_file;
    const char	*username_file;
    const char	*global_filter;
    const char	*users_filter;
    const char	*reports_logo;
    const char	*configuration_name;
    const char	*email_address;
    const char	*locale;
    const char	*reports_privacy_username;
    const char	*users_excess;
    double	reports_svg_width;
    double	users_excess_limit;
    time_t	real_time_timeout;
    time_t	reports_rotate;
    time_t	log_time_zone_shift;
    short int	log_format;
    short int	reports_url_limit;
    short int	top_sites_limit;
    short int	reports_overwrite;
    short int	reports_privacy_mode;
    char	log_skip_errors;
    char	reports_bytes_divisor;
    char	index_show_calendar;
    char	index_sort;
    char	server_efficiency_bytes_divisor;
    char	username_is_preferred;
    char	username_unescape;
    char	username_resolve_ip;
    char	users_graphics_svg;
    char	reports_show_info;
    char	reports_indicators;
    char	users_fullurl_report;
    char	users_fullurl_split;
    char	users_report;
    char	top_sites_report;
    char	top_sites_svg;
    char	top_users_svg;
    char	server_efficiency_report;
    char	server_efficiency_svg;
    char	recipient_tolower;
} Config;

typedef struct owConfig {
    const char  *target_directory;
    const char	*log_file;
    const char	*email_address;
    const char	*configuration_name;
} owConfig;

typedef struct iConfig {
    const char	*locDivisor;
    const char	*ReportDir;
    const char	*CharSet;
    const char	*locFile;
    const char	*TTLname;
    const char	*AVGname;
    const char	*DFname;
    const char	*IRname;
    const char	*LRname;
    const char	*TURname;
    const char	*TSRname;
    const char  *ERname;
    const char	*SERname;
    const char	*PTname;
    short int	SYear;
    short int	EYear;
    short int	LFcount;
    short int	LFcsscount;
    time_t	SDate;
    time_t	EDate;
    short int	LocDateLen;
    short int	dldCount;
    short int	utCount;
    short int	ueCount;
    double	reports_bytes_divisor;
    char	**Downloads;
    struct SAusertab *Usertab;
    char	Period[DATETIMESIZE * 2L];
    char	*(*SAB)(double);
    char	GF;
    char	UF;
    char	Mode;
    char	PScale;
} iConfig;

typedef struct Localisation {
    const char	*No;
    const char	*User;
    const char	*Connections;
    const char	*Seconds;
    const char	*Bytes;
    const char	*URL;
    const char	*Period;
    const char	*SortFieldNOrder;
    const char	*Users;
    const char	*Served;
    const char	*Actual;
    const char	*Cached;
    const char	*Denied;
    const char	*TrafType;
    const char	*OthLocal;
    const char	*TopUsers;
    const char	*TopSites;
    const char	*SEfficiency;
    const char	*InRealTime;
    const char	*DnT;
    const char	*DsT;
    const char	*DoM;
    const char	*Asc;
    const char	*Desc;
    const char	*TOTAL;
    const char	*AVERAGE;
    const char	*MAXIMUM;
    const char	*Code;
    const char	*GeneratedBy;
    const char	*Configuration;
    const char	*GenerationD;
    const char	*PSP;
    const char	*P;
    const char	*T;
    const char	*G;
    const char	*M;
    const char	*k;
    const char	*Sender;
    const char	*Messages;
    const char	*Recipient;
    const char	*Senders;
    const char	*Recipients;
    const char	*Month;
    const char	*Year;
} Localisation;

typedef struct FConfig {
    short int	inameCount;
    short int	urlCount;
    short int	ipCount;
    short int	codeCount;
    short int	limit;
    const void	**inames;
    const void	**urls;
    const void	**ips;
    short int	*codes;
    char	*methods;
    char	*name;
    double	bytes;
    char	stats;
    char	inameMode;
    char	urlMode;
    char	ipMode;
    char	codeMode;
    char	methodMode;
    char	statMode;
    char	bytesMode;
    char	showbytes;
} FConfig;

typedef struct SAFNAMES {
    char        srt[FILENAMESIZE];
    char        uns[FILENAMESIZE];
} SAFNAMES;

typedef struct SVGConfig {
    double		Scale;
    short int		Top;
    char		Legend;
    char		Pie;
    char		Relative;
} SVGConfig;

typedef struct RConfig {
    double		SVGwidth;
    double		SVGheight;
    char		*UFRsort;
    char		*URsort;
    char		*UDRsort;
    char		*LFRsort;
    char		*IRsort;
    struct SAFNAMES	TSRfnames;
    struct SVGConfig	UGR;
    struct SVGConfig	TSGR;
    struct SVGConfig	TUGR;
    struct SVGConfig	SEGR;
} RConfig;


extern struct Config cfg;
extern struct owConfig owcfg;
extern struct iConfig icfg;
extern struct Localisation defloc;
extern struct Localisation *loc;
extern struct FConfig *gfcfg;
extern struct FConfig *ufcfg;
extern struct FConfig **lfcfgs;
extern struct RConfig rcfg;


extern void InitConfigWithCL(int argc, char **argv);
extern void InitConfig(void);
extern void InitLocale(void);


#define	LOCSIZE	sizeof(Localisation)/sizeof(char *)
