#include "global.h"


int compCDentry(const struct ChartData *el1, const struct ChartData *el2);
int compCDentry(const struct ChartData *el1, const struct ChartData *el2) {
    return SAnumcmp(el1->value, el2->value);
}


int SSVGcountnmax(struct ChartData *icdata, double *maxvalue);
int SSVGcountnmax(struct ChartData *icdata, double *maxvalue) {
    int retval = 0;

    *maxvalue = 0.0;
    for (; icdata[retval].label != NULL; retval++) *maxvalue = SAmax(*maxvalue, icdata[retval].value);

    return retval;
}


int SSVGcountnsum(struct ChartData *icdata, double *sumvalue);
int SSVGcountnsum(struct ChartData *icdata, double *sumvalue) {
    int retval = 0;

    *sumvalue = 0.0;
    for (; icdata[retval].label != NULL; retval++) *sumvalue += icdata[retval].value;

    return retval;
}


void SSVGdrawpie(FILE *GR, struct ChartData *icdata, struct SVGConfig *iGR) {
    int count, i, j;
    double c_y0, c_x0, c_R, c_lfontsize, c_pfontsize, c_fontindent, c_lfontindent, c_pfontindent, c_pi, 
           c_linewidth, c_barwidth, c_ibspace, percent, sxpos, sypos, expos, eypos, pxpos, pypos, lxpos, 
           lypos, rad, tmp, sumvalue, sum = 0.0;
    const char *label, *llabel = "*";


    /* compute max value and count */
    count = SSVGcountnsum(icdata, &sumvalue);
    c_pi = 2.0 * SSVGPI / sumvalue;				/* this is useful constant			*/
    c_lfontsize = rcfg.SVGheight / FSDIV * PC_FSCALE / iGR->Scale;/* size of label font				*/
    c_pfontsize = rcfg.SVGheight / FSDIV * PC_FSCALE / 2.0;	/* size of percent font				*/
    c_R = rcfg.SVGheight / 2.0 - c_lfontsize * (3.0 * TINDENT + 2.0);/* radius of circle (2 chars and 3 indents)*/
    if (iGR->Legend) c_x0 = rcfg.SVGwidth / 3.0;		/* x of circle center				*/
    else c_x0 = rcfg.SVGwidth / 2.0;
    c_y0 = rcfg.SVGheight / 2.0 - c_lfontsize * (2.0 * TINDENT + 1.0);/* y of circle center (1 char and 2 indents)*/
    c_linewidth = rcfg.SVGheight / LWDIV;			/* width of line				*/
    c_fontindent = c_lfontsize * TINDENT;			/* label font indent				*/
    c_lfontindent = c_R + c_fontindent;				/* label font indent from circle center		*/
    c_pfontindent = c_R - c_pfontsize / TINDENT / 2.0;		/* percent font indent from circle center	*/

    qsort(icdata, (size_t)count, sizeof(struct ChartData), (int (*)(const void *, const void *))compCDentry);

    /* initial values */
    sxpos = c_x0 + c_R;
    sypos = c_y0;
    rad = 0.0;

    for (i = 0; i < count && icdata[i].value; i++) {
	tmp = c_pi * icdata[i].value;
	
	/* sectors calculations */
	rad += tmp;
	/* not show pies smaller than PC_MAXPIE * 100% */
	if (icdata[i].value / sumvalue < PC_MAXPIE && i != count - 1) {
	    rad = 0.0;						/* no space for other sectors */
	    percent = 100.0 - sum;
	    label = llabel;
	} else {
	    percent = icdata[i].value * 100.0 / sumvalue;
	    sum += percent;
	    label = icdata[i].label;
	}
	expos = c_x0 + cos(rad) * c_R;
	eypos = c_y0 + sin(rad) * c_R;
	/* labels calculations */
	tmp = rad - tmp / 2.0;
	lxpos = c_x0 + cos(tmp) * c_lfontindent;
	lypos = c_y0 + sin(tmp) * c_lfontindent;
	if (lypos > c_y0) lypos += c_lfontsize;

	pxpos = c_x0 + cos(tmp) * c_pfontindent;
	pypos = c_y0 + sin(tmp) * c_pfontindent;
	if (pypos > c_y0) pypos += sin(tmp) * c_pfontsize;

	/* workaround */
	if (percent == 100.0) eypos -= 0.0001;

	/* draw sectors and labels */
	SAfprintf(GR, "\
\t\t<path class=\"c%d\" d=\"M%lf,%lf L%lf,%lf A%lf,%lf 0 %hd,1 %lf,%lf z\" stroke-width=\"%lf\"/>\n\
\t\t<text class=\"%ctext\" x=\"%lf\" y=\"%lf\" font-size=\"%lf\">%s</text>\n\
\t\t<text x=\"%lf\" y=\"%lf\" font-size=\"%lf\">%.1lf%%</text>\n",
		(label == llabel) ? 0 : (i % 7) + 1, c_x0, c_y0, sxpos, sypos, c_R, c_R, (percent > 50.0) ? 1 : 0, expos, eypos, c_linewidth,
		(lxpos > c_x0) ? 's' : 'e', lxpos, lypos, c_lfontsize, label,
		pxpos, pypos, c_pfontsize, percent);
		
	if (rad == 0.0) break;
	
	sxpos = expos;
	sypos = eypos;
    }

    /* draw summary label */
    SAfprintf(GR, "\
\t\t<text x=\"%lf\" y=\"%lf\" font-size=\"%lf\">100%% = %s</text>\n",
	    c_x0, rcfg.SVGheight - c_fontindent, c_lfontsize, SAB3(sumvalue));

    /* draw legend */
    if (iGR->Legend) {
	c_barwidth = c_lfontsize;
	c_ibspace = c_fontindent;
	lxpos = rcfg.SVGwidth / 3.0 * 2.0 + c_barwidth;
	sum = 0.0;
	for (j = 0; j <= i; j++) {
	    if (j == i) {
		/* signbit required because there could be -0.0 */
		if (!(percent = 100.0 - sum) || signbit(percent)) break;
		label = llabel;
		tmp = sumvalue * percent / 100.0;
	    } else {
		percent = icdata[j].value * 100.0 / sumvalue;
		sum += percent;
		label = icdata[j].label;
		tmp = icdata[j].value;
	    }
	    lypos = c_ibspace + (c_ibspace + c_barwidth) * (double)j;
	    SAfprintf(GR, "\
\t\t<rect class=\"c%d\" x=\"%lf\" y=\"%lf\" width=\"%lf\" height=\"%lf\" stroke-width=\"%lf\" />\n\
\t\t<text class=\"stext\" x=\"%lf\" y=\"%lf\" font-size=\"%lf\">%s (%s) %.1lf%%</text>\n",
		    (label == llabel) ? 0 : (j % 7) + 1, lxpos, lypos, c_barwidth, c_barwidth, c_linewidth,
		    lxpos + c_barwidth + c_ibspace, lypos + c_barwidth - c_fontindent, c_lfontsize, label, SAB3(tmp), percent);
	    }
    }
}


void SSVGdrawbars(FILE *GR, struct ChartData *icdata, struct SVGConfig *iGR) {
    int count, i = 0, j, k;
    double c_barwidth, c_linewidth, c_fontsize, c_yaxistep, c_ibspace, 
	   c_fontindent, c_halfbar, c_xshift, c_yshift, 
	   xpos, ysize, maxvalue, tmp;


    if (!(count = SSVGcountnmax(icdata, &maxvalue))) return;
    /* compute operational values */
    c_linewidth = rcfg.SVGheight / LWDIV;				/* width of line		*/
    c_fontsize = rcfg.SVGwidth / FSDIV;					/* size of font			*/
    c_fontindent = c_fontsize * TINDENT;				/* indent of font from object	*/
    c_yshift = rcfg.SVGheight - c_fontsize - c_fontindent * 2.0;	/* y axis shift			*/
    c_xshift = c_fontsize * FONTPROP + c_fontindent * 2.0;		/* x axis shift			*/
    c_ibspace = (BC_IBSPDIV + BC_IBSPDIV / (double)count);		/* inter-bar space		*/
    c_barwidth = (rcfg.SVGwidth - c_xshift) / ((double)count + c_ibspace * (double)(count + 1));/* width of bar	*/
    c_halfbar = c_barwidth / 2.0;					/* half of bar width		*/

    /* compute steps of lines at y axis */
    if (iGR->Relative) for (i = 0; pow(10.0, (double)i) < maxvalue; i++);
    else i = 14;
    c_yaxistep = c_yshift / (double)(i + 1);
    /* draw y axis lines and labels */
    for (j = 0;; j++)
	for (k = 0; k < 3; k++) {
	    if ((tmp = pow(1024.0, (double)j) * pow(10.0, (double)k)) == 1.0) continue;
	    ysize = c_yshift - c_yaxistep * log10(tmp);
	    SAfprintf(GR, "\
\t\t<path d=\"M%lf,%lf L%lf,%lf\" stroke-width=\"%lf\" />\n\
\t\t<text x=\"%lf\" y=\"%lf\" class=\"etext\" font-size=\"%lf\">%s</text>\n", 
		    c_xshift + c_linewidth, ysize, rcfg.SVGwidth, ysize, c_linewidth,
		    c_xshift - c_fontindent, ysize, c_fontsize, SAB3(tmp));
	    if (tmp > pow(10.0, (double)i)) goto end;
	}
end:
    /* draw bars with labels */
    for (i = 0; i < count; i++) {
	xpos = c_xshift + c_barwidth * (c_ibspace + (1.0 + c_ibspace) * (double)i);

	/* label */
	SAfprintf(GR, "\
\t\t<text x=\"%lf\" y=\"%lf\" font-size=\"%lf\">%s</text>\n",
		  xpos + c_halfbar, c_yshift + c_fontindent + c_fontsize, c_fontsize, icdata[i].label);
	
	if (icdata[i].value) ysize = c_yaxistep * log10(icdata[i].value);
	else continue;
	
	/* bar and value on bar top */
	SAfprintf(GR, "\
\t\t<rect x=\"%lf\" y=\"%lf\" width=\"%lf\" height=\"%lf\" stroke-width=\"%lf\" />\n\
\t\t<text x=\"%lf\" y=\"%lf\" font-size=\"%lf\">%s</text>\n",
	    xpos, c_yshift - ysize, c_barwidth, ysize, c_linewidth,
	    xpos + c_halfbar, c_yshift - ysize - c_fontindent, c_fontsize, SAB3(icdata[i].value));
    }

    /* draw x/y axes */
    SAfprintf(GR, "\
\t\t<path d=\"M%lf,%lf L%lf,%lf L%lf,%lf\" class=\"aline\" stroke-width=\"%lf\" />\n", 
	    c_xshift, 0.0, c_xshift, c_yshift, rcfg.SVGwidth, c_yshift, c_linewidth * BC_AWCOEF);
}


void SVGObject(FILE *GRparent, const char *name) {
    SAfprintf(GRparent, "\
\t<object type=\"image/svg+xml\" data=\"g_%s.svg\" width=\"%.0f\" height=\"%.0f\"></object>\n",
	      name, rcfg.SVGwidth, rcfg.SVGheight);
}


void SVGReport(FILE *GRparent, struct ChartData *icdata, struct SVGConfig *iGR, const char *name, const char mode) {
    FILE *GR;


    SAsetlocale(LC_NUMERIC, "C");
    GR = SAfopen("w", "%s/g_%s.svg", icfg.ReportDir, name);

    SAfprintf(GR, "\
<?xml version=\"1.0\" encoding=\"%s\"?>\n\
<?xml-stylesheet href=\"../gsa.css\" type=\"text/css\"?>\n\
<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n\
\t<svg viewBox=\"0 0 %.0f %.0f\" xmlns=\"http://www.w3.org/2000/svg\" version=\"1.1\" xmlns:xlink=\"http://www.w3.org/1999/xlink\">\n",
	      icfg.CharSet, rcfg.SVGwidth, rcfg.SVGheight);
    if (iGR->Pie) SSVGdrawpie(GR, icdata, iGR);
    else SSVGdrawbars(GR, icdata, iGR);
    SAfwrite1("</svg>\n", 7L, GR);

    SAfclose(GR);
    SAsetlocale(LC_NUMERIC, cfg.locale);

    if (mode) SVGObject(GRparent, name);


    free(icdata);
}


void SVGReportAnyGraphs(double *mBytes, const char *vname, const char *name, short int elcount) {
    FILE *UGR;
    struct ChartData *UGRCD;
    char *UGRN;
    short int i, k = (elcount == 24) ? 0 : 1;


    UGR = SAfopenH("../", 'g', vname, NULL, 1, "%s/g_%s.html", icfg.ReportDir, name);

    UGRCD = (struct ChartData *)SAmalloc(((size_t)(elcount + 1)) * sizeof(struct ChartData));
    UGRN = (char *)SAmalloc(((size_t)elcount) * sizeof(char) * 3);
    for (i = 0; i < elcount; i++) {
	UGRCD[i].label = &UGRN[i * 3L];
	SAsnprintf(UGRCD[i].label, 3L, "%hd", i + k);
	UGRCD[i].value = mBytes[i];
    }
    UGRCD[i].label = NULL;

    SVGReport(UGR, UGRCD, &rcfg.UGR, name, 1);

    free(UGRN);
    SAfcloseH(UGR);
}
